// $Id: buffer.h,v 1.6 2006/06/24 09:39:28 dvermeir Exp $
#ifndef	DV_THREAD_BUFFER_H
#define	DV_THREAD_BUFFER_H

#include <dvthread/lock.h>

#define TRACE std::cerr << pthread_self() << ":" << __FILE__ << "." << __LINE__ << std::endl;

// This class represents a buffer with synchronized access.
class Buffer: public Dv::Thread::Monitor {
  public:	
    Buffer() throw (std::runtime_error): Dv::Thread::Monitor("buffer",2), // &std::cerr),
      n_items_(0) {}
    // Add an item to a buffer.
    void put(int i) throw (std::runtime_error) {
      Dv::Thread::Lock lock(*this, "put", 0); //  &std::cerr); // Get exclusive access to *this.
      while (n_items_==MAX)
        if (!wait(OK_TO_PUT,2000)) // Wait at most 2 secs.
          throw std::runtime_error("Buffer::put() timed out");
      std::cerr << "put " << i << " ";
      data_[n_items_++] = i; // Actually put the item.
      signal(OK_TO_GET);
    }
  
     // Obtain and remove the last item in the buffer.
    int get() throw (std::runtime_error) {
      Dv::Thread::Lock lock(*this, "get"); // , &std::cerr); // Get exclusive access to *this.
      while (n_items_ == 0) {
        if (!wait(OK_TO_GET, 2000)) 
          throw std::runtime_error("Buffer::get() timed out");
      }
      int tmp = data_[--n_items_];
      std::cerr << "get " << tmp << " ";
      signal(OK_TO_PUT);
      return tmp;
    }

  private:
    // Buffer capacity.
    enum { MAX = 3 };
    // Names for conditions.
    enum { OK_TO_GET = 0, OK_TO_PUT = 1 }; /// conditions 
    // Number of items in the buffer.
    int n_items_;
    // Actual store for items.
    int data_[MAX];
};
#endif
