#ifndef	DV_THREAD_READER_H
#define	DV_THREAD_READER_H

#include <unistd.h>
#include <dvthread/thread.h>
#include "buffer.h"


// Reader thread class. A reader simply retrieves items from a buffer.
class Reader: public Dv::Thread::Thread {
  public:
    // Constructor; n is the number of items to retrieve.
    Reader(Buffer& buf, unsigned int n): Thread(), buffer_(buf), n_(n) {}
    // The function executed by this tread.
    virtual int main() {
      try {
        std::cerr << "reader main, id = " << id() << std::endl;
        for (unsigned int i=0;(i<n_);++i)
          buffer_.get();
      }
      catch (std::exception& e) {
        std::cerr << "reader exception: " << e.what() << std::endl;
        return 1;
      }
      return 0;
    }
    // Destructor. Wait for this thread to finish before we detroy it.
    virtual ~Reader() { std::cerr << "~Reader" << std::endl; } 
  private:
    // Buffer from where items will be retrieved.
    Buffer&	buffer_;
    // Number of items to retrieve.
    unsigned int  n_;
};
#endif
