// $Id: test-delete-after-main.C,v 1.2 2006/06/24 09:39:28 dvermeir Exp $

/** Program testing delete after main feature. */

#include <string>
#include <iostream>
#include <stdexcept>
#include <unistd.h> // sleep, close
#include <dvthread/thread.h>


class TestThread: public Dv::Thread::Thread {
  public:
    // Factory member function to prohibit constructing without new.
    static TestThread* make() { return new TestThread(); }
  
    ~TestThread() { ++destructor_called; }
  
    static int destructor_called;
    static int dead;
  
    int main() {
      try {
        std::string line;
        while (!dead && getline(std::cin, line) ) 
          std::cout << "input:" << std::endl;
        return 0;
      }
      catch (std::exception& e) {
        std::cerr << e.what() << std::endl;
        return 1;
      }
    }
  
    void killed() throw () {
      ::close(0);
      ++dead;
    }
  
  private:
    TestThread(): Thread(true) {
      // detach() would be usual, but not here since we want to join()
      // the thread for testing purposes
    }
};

int TestThread::dead = 0;
int TestThread::destructor_called = 0;

int
main(int,char**) {
  try {
    // Dv::Thread::Thread::log = &std::cerr;
    TestThread* t(TestThread::make());
  
    t->start();
    sleep(1);
    t->kill();
    t->join();
    if (TestThread::dead == 0) 
      throw std::runtime_error("kill() failed");
    if (TestThread::destructor_called==0) 
      throw std::runtime_error("destructor not called automatically");
  }
  catch (std::exception& e) {
    std::cerr << "error: " << e.what() << std::endl;
    return 1;
  }
  return 0;
}
